package  
{
	import net.flashpunk.World;
	import flash.utils.ByteArray;
	import net.flashpunk.*;
	import GameWorld;
	import Player;
	import Item;
	import Background;
	import Turret;
	//import Particles;
	
	/**
	 * ...
	 * @author Seth A. Robinson
	 */
	public class Level extends Entity
	{
		public var level:XML;
	
		public const FOLLOW_TRAIL:Number = 50;
		public const FOLLOW_RATE:Number = .9;
	
		[Embed(source = '../assets/RT_MapEd/maps/Level1/seth_tiles.png')] private static const TILES:Class;
	
		[Embed(source = 'level1.oel', mimeType = 'application/octet-stream')] private static const LEVEL1:Class;
		[Embed(source = 'level2.oel', mimeType = 'application/octet-stream')] private static const LEVEL2:Class;
		[Embed(source = 'level3.oel', mimeType = 'application/octet-stream')] private static const LEVEL3:Class;
		[Embed(source = 'level4.oel', mimeType = 'application/octet-stream')] private static const LEVEL4:Class;
		[Embed(source = 'level5.oel', mimeType = 'application/octet-stream')] private static const LEVEL5:Class;
		
		public var levelWidth:uint;
		public var levelHeight:uint;
	
		public function Level(levNum:int) 
		{
			super();
			
			var file:ByteArray = null;
			
			if (levNum == 1)
			{
				file = new LEVEL1;
				var sound:Sfx = new Sfx(GC.SFX_LEVEL1);
				sound.play();
				FP.world.add(new Background(GC.GFX_BACKGROUND1));
		
			} else if (levNum == 2)
			{
				file = new LEVEL2;
						FP.world.add(new Background(GC.GFX_BACKGROUND2));
	
		} else if (levNum == 3)
			{
				file = new LEVEL3;
							FP.world.add(new Background(GC.GFX_BACKGROUND3));
	
		    }  else if (levNum == 4)
			{
				file = new LEVEL4;
							FP.world.add(new Background(GC.GFX_BACKGROUND1));
	
		    } else if (levNum == 5)
			{
				file = new LEVEL5;
							FP.world.add(new Background(GC.GFX_BACKGROUND2));
	
		    }
			level = XML(file.readUTFBytes(file.length));
			levelWidth = level.width;
			levelHeight = level.height;
		
			//scan xml and add a Floor() object for each layer.  Floor() will figure out if it's a tile or collision layer
			for each (var p:XML in level.*)
			{
		//		trace("Found " + p.name() + " of kind " + p.*.length());
				var childCount:int = p.*.length();
				if (p.name() == "tiles" || p.name() == "objects" || childCount > 1)
				{
					trace("Adding layer "+p.name());
					FP.world.add(new Floors(p, levelWidth, levelHeight, TILES));
				}
			}
		
			for each (var p:XML in level.objects[0].player)
			{
				var player:Player = new Player();
				GameWorld.player = player;
				FP.world.add(player);
				player.Init(p.@x, p.@y);
			}
		
				for each (var p:XML in level.objects[0].food)
			{
		
				var item:Item = new Item();
				FP.world.add(item);
				var n:int = p.@export_num;
				item.SetType("food", n);
				item.x = p.@x;
				item.y = p.@y;
			}
		
			for each (var p:XML in level.objects[0].exit)
			{
		
				var item:Item = new Item();
				item.SetType("exit", 0);
				FP.world.add(item);
				item.x = p.@x;
				item.y = p.@y;
			}

			for each (var p:XML in level.objects[0].text)
			{
			
				var item:Item = new Item();
				item.SetType("text", 0);
				FP.world.add(item);
				item.x = p.@x;
				item.y = p.@y;
				item.SetText(p.@text);
			}
			
				for each (var p:XML in level.objects[0].trigger)
			{
		
				var item:Item = new Item();
				item.SetType("trigger", 0);
				FP.world.add(item);
				item.x = p.@x;
				item.y = p.@y;
				item.SetTrigger(p.@type);
				
			}

	
				for each (var p:XML in level.objects[0].enemy)
			{
		
				var s:String = p.@type;
				var t:Turret = new Turret(s);
				t.x = p.@x;
				t.y = p.@y;
				FP.world.add(t);
				
			}
			
			
			//add(new Background(1));
			//add(new JetBar);
			
		
			trace("Level initted");
		}
		
			override public function update():void
		{
			cameraFollow();
		}
		
		private function cameraFollow():void
		{
			
			// make camera follow the player
			FP.point.x = FP.camera.x - targetX;
			FP.point.y = FP.camera.y - targetY;
			
			var dist:Number = FP.point.length;
				
			if (dist > FOLLOW_TRAIL) dist = FOLLOW_TRAIL;
			FP.point.normalize(dist * FOLLOW_RATE);
			
			
			FP.camera.x = int(targetX + FP.point.x);
			FP.camera.y = int(targetY + FP.point.y);
			
			// keep camera in room bounds
			FP.camera.x = FP.clamp(FP.camera.x, 0, levelWidth - FP.width);
			FP.camera.y = FP.clamp(FP.camera.y, 0, levelHeight - FP.height);
			
		}
		
		/**
		 * Getter functions used to get the position to place the camera when following the player.
		 */
		private function get targetX():Number
		{
			var p:Player = GameWorld.player as Player;
			
			return (p.centerX )  - FP.width / 2; 
		}
		
		private function get targetY():Number 
		{
			var p:Player = GameWorld.player as Player;
			
			return (p.centerY )  - FP.height / 2; 
		}
	}

}