package  
{
	import flash.geom.Point;
	import flash.media.Sound;
	import net.flashpunk.graphics.Emitter;
	import net.flashpunk.graphics.Graphiclist;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.ParticleType;
	import net.flashpunk.tweens.misc.NumTween;
	import net.flashpunk.tweens.motion.LinearMotion;
	import net.flashpunk.utils.Ease;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.*;
	import net.flashpunk.graphics.Spritemap;
	
	import Particles;
	import Item;
	import GC;
	/**
	 * ...
	 * @author Seth A. Robinson
	 */
	
	 
	 public class Player extends Moveable
	{
		[Embed(source = '../assets/cat.png')]  private const PLAYER:Class;
		public var image:Spritemap=  new Spritemap(PLAYER, 128, 64);
		public static var getPlayer:Player = null;
		public var onSolid:Boolean;
		public var spdX:Number = 0;
		public var spdY:Number = 0;
		public const MAXX:Number = 300;
		public const MAXY:Number = 400;
		public const MAXYUP:Number = -600;
		public const GRAV:Number = 1500;
		public const FLOAT:Number = 3000;
		public const ACCEL:Number = 2500;
		public const DRAG:Number = 1800;
		public const JUMP:Number = -500;
		public const LEAP:Number = 1.5;
		public const JETPACKPOW:Number = -2300;
		public const SCALE:LinearMotion = new LinearMotion;
		public const ROTATE:NumTween = new NumTween;
	
		public var emitter:Emitter;
		private var lastSavePosX:Number;
		private var lastSavePosY:Number;
		
		public var canJetPack:Boolean;
		
		public var jetPackPower:Number;
		public var jetPackPercent:Number;
		public var gameScale:Number;
		public var hitBoxModX:Number;
		public var hitBoxModY:Number;
		public var weaponLevel:int = 0;
		var soundJump1:Sfx;
		
		public function Player() 
		{
			getPlayer = this;
			canJetPack = false;
			
			Input.define("L", Key.LEFT);
			Input.define("R", Key.RIGHT);
			Input.define("JUMP", Key.SPACE, Key.SHIFT, Key.CONTROL, Key.UP, Key.X, Key.A, Key.S);
			Input.define("FIRE", Key.Z);
		
			image.add("stand", [0], 5, true);
			image.add("run", [0,1], 5, true);
			image.add("jump", [2], 5, true);
			
			graphic = image;
			image.smooth = true;
			
			
			gameScale = 0.5;
			//setHitboxTo(image);
			addTween(SCALE);
			addTween(ROTATE);
			SCALE.x = SCALE.y = 1;
			
			SetGameScale(gameScale);
			soundJump1 = new Sfx(GC.SFX_JUMP1);
			//centerOrigin();
			
			trace("Initting player");
		}
		public function OnModSize(mod:Number):void
		{
			var n:Number = gameScale + (gameScale * mod);
			y +=  ((image.height * gameScale) - (image.height * n)) - 1;
			lastSavePosX = x;
			lastSavePosY = y;
			
			SetGameScale(n);
		}
		public function SetGameScale(scale:Number):void
		{
			hitBoxModX = 0.80;
			hitBoxModY = 0.80;
		
			gameScale = scale;
			setHitbox(image.width * gameScale * hitBoxModX, image.height * gameScale * hitBoxModY );
			
			setOrigin( ((image.width * gameScale * hitBoxModX) -(image.width * gameScale)) / 2,
			((image.height * gameScale * hitBoxModY) -(image.height * gameScale)) / 2);
		}
		public function Init(startx:Number, starty:Number):void
		{
			x = startx;
			y = starty;
			lastSavePosX = x;
			lastSavePosY = y;
		
		}
		override public function added():void 
		{
			emitter = (FP.world.classFirst(Particles) as Particles).emitter;
		}
		
		private function DieAndRestart():void
		{
				x = lastSavePosX;
				y = lastSavePosY;
				
		var r:int = FP.rand(3);
		var sfx:Sfx;
	
		trace(r);
			switch (r)
			{
				case 0:  sfx = new Sfx(GC.SFX_MEOW1); break;
				case 1:  sfx = new Sfx(GC.SFX_MEOW2); break;
				case 2:  sfx = new Sfx(GC.SFX_MEOW3); break;
				case 3:  sfx = new Sfx(GC.SFX_MEOW4); break;
			}
		sfx.play();
		}
				
		override public function update():void
		{
			check_fire();
			checkFloor();
			gravity();
			acceleration();
			jumping();
			move(spdX * FP.elapsed, spdY * FP.elapsed);
			animation();
			check_special_tiles();
		}
	
		private function check_special_tiles():void
		{
			var t:Turret =  collide(damage, x, y + 1) as Turret;
			
			if (t)
			{
				if (gameScale < 2)
				{
					DieAndRestart();
				} else
				{
					
					t.OnExplode();
					FP.world.remove(t);
				}
			}
		
			
				var b:Bullet =  collide("bullet", x, y + 1) as Bullet;
			
				if (b)
				{
						FP.world.remove(b);
				
						if (gameScale < 2)
						{
							DieAndRestart();
						} else
						{
							(FP.world as GameWorld).ScheduleSound(0, GC.SFX_TINK);

						}
				}
			
			
				var item:Item = collide(food, x, y) as Item;
				
				if (item)
				{
					item.OnTouched(this);
				}
				
				var item:Item = collide("trigger", x, y) as Item;
				
				if (item)
				{
					item.OnTouched(this);
				}
				
				item = collide("exit", x, y) as Item;
				
				if (item)
				{
					active = false;
					item.OnTouchedExit(this);
				}
			
		}
		
		
		public function OnGiveWeapon()
		{
			weaponLevel = 1;
		}
		private function checkFloor():void
		{
					if (collide(solid, x, y + 1))
					{
						onSolid = true;
					}
			else onSolid = false;
		}
	
		private function check_fire():void
		{
			
			if (weaponLevel == 0) return;
			
				if (Input.pressed("FIRE"))
				{
				trace("fire!");	
				var p:Point = new Point();
				p.x = 1;
				p.y = 0;
				if (image.flipped) p.x *= -1;
				var b:Bullet = new Bullet();
				b.SetAim(p.x, p.y);
				b.x = x+image.scaledWidth/2;
				b.y = y+image.scaledHeight/2;
				b.SetOwner(this);
				
				FP.world.add(b);
				}
		}
		
		private function gravity():void
		{
			if (onSolid) return;
			var g:Number = GRAV;
			//if (spdY < 0 && !Input.check("JUMP")) g += FLOAT;
			spdY += g * FP.elapsed;
			if (spdY > MAXY) spdY = MAXY;
			
			if (spdY < MAXYUP) spdY = MAXYUP;
			
		}
		
			private function acceleration():void
		{
			// evaluate input
			var accel:Number = 0;
			if (Input.check("R")) accel += ACCEL;
			if (Input.check("L")) accel -= ACCEL;
			
			// handle acceleration
			if (accel != 0)
			{
				if (accel > 0)
				{
					// accelerate right
					if (spdX < MAXX)
					{
						spdX += accel * FP.elapsed;
						if (spdX > MAXX) spdX = MAXX;
					}
					else accel = 0;
				}
				else
				{
					// accelerate left
					if (spdX > -MAXX)
					{
						spdX += accel * FP.elapsed;
						if (spdX < -MAXX) spdX = -MAXX;
					}
					else accel = 0;
				}
			}
			
			// handle decelleration
			if (accel == 0)
			{
				if (spdX > 0)
				{
					spdX -= DRAG * FP.elapsed;
					if (spdX < 0) spdX = 0;
				}
				else
				{
					spdX += DRAG * FP.elapsed;
					if (spdX > 0) spdX = 0;
				}
			}
		}

		public function GetJetpackPercent():Number
		{
			return 0.3;
		}
		private function jumping():void
		{
			var i:int;
			
			
		//	trace("Speedy is " + spdY);
			
			if (onSolid && Input.pressed("JUMP"))
			{
				soundJump1.play();
			//	trace("Gamescale: " + gameScale);
				var jumpPower:Number = JUMP * (0.5 + (gameScale * gameScale * gameScale));
				if (jumpPower > -500) jumpPower = -500;
					if (jumpPower < -4000) jumpPower = -4000;
			
				spdY = spdY + jumpPower;
				onSolid = false;
				//if (spdX < 0 && image.flipped) spdX *= LEAP;
				//else if (spdX > 0 && !image.flipped) spdX *= LEAP;
				
				SCALE.setMotion(1, 1.2, 1, 1, .2, Ease.quadIn);
				//ROTATE.tween(0, 360 * -FP.sign(spdX), FP.scale(Math.abs(spdX), 0, MAXX, .7, .5), Ease.quadInOut);
				
				//var i:int = 10;
				//while (i --) emitter.emit("dust", x - 10 + FP.rand(20) , y + 16);
				
				/*
				for (i = 0; i < 20; i++)
				{
				//emitter.emit("trail", x - 10 + FP.rand(20), y + 20 + FP.rand(20));
					emitter.emit("dust", x - 10 + FP.rand(20) , y + 30);
				}
				*/
			
				canJetPack = false;
			
				
			} else
			{
			
				if (Input.released("JUMP"))
				{
					canJetPack = true;
				}
			}
		}
		
		override protected function collideX(e:Entity):void 
		{
			//if (spdX > 100 || spdX < -100) SCALE.setMotion(1, 1.2, 1, 1, .2, Ease.quadIn);
			spdX = 0;
		}
		
		/**
		 * Vertical collision handler.
		 */
		override protected function collideY(e:Entity):void 
		{
			if (spdY > 0)
			{
			//	SCALE.setMotion(1.2, 1, 1, 1, .2, Ease.quadIn);
			//	spdY = 0;
			spdY = 0;
			}
			else
			{
				//SCALE.setMotion(1.2, 1, 1, 1, .1, Ease.quadOut);
				spdY = 0;
			}
		}

			private function animation():void
		{
			
			if (onSolid)
			{
				if (spdX == 0)
				{
					image.play("stand");
				} else
				{
					image.play("run");
				}
				
			} else {
				//in air
				image.play("jump");
			}
			
			// control facing direction
			if (spdX != 0) image.flipped = spdX < 0;
			
			// image scale tweening
			image.scaleX = SCALE.x*gameScale;
			image.scaleY = SCALE.y*gameScale;
			
			
			// image rotation
			if (onSolid)
			{
				image.angle = 0;
				ROTATE.active = false;
				ROTATE.value = 0;
			}
			else 
			{
				//image.angle = (spdX / MAXX) * 10 + ROTATE.value;
			}
		}
		
		
	}

}